unit uConfig;
{$INCLUDE colbuild.inc}

interface

uses
  SysUtils, Classes, IniFiles,
  uFunc;

type
  TReaderMemoryType = (mtInternal, mtMS, mtSD);
  TEncoding = (enOEM, enANSI, enUnicode, enUTF8);
  
  TConfig = class
    private
      fCalibrePath: string;
      fGenerateMetadata: boolean;
      fRefreshExisting: boolean;
      fMemoryType: TReaderMemoryType;
      fRecursive: boolean;
      fFileNameRegExp: string;
      fPlaylistFN, fCacheInFN, fCacheOutFN: string;
      fBooksPath, fBooksBasePath: string;
      fTimeDelta: TDateTime;
      fCollectionRegexp, fCollectionRegexpTitle: string;
      fLRFMeta, fPDFMeta, fRTFMeta, fTXTMeta, fEPUBMeta: string;
      fConvertToUTF: boolean;
      fEncoding: TEncoding;
      function GetMemoryType(const MemType: string; Default: TReaderMemoryType): TReaderMemoryType;
    public
      constructor Create; virtual;
      procedure Init; virtual;
      procedure LoadFromIniFile(const IniFileName, Section: string);
      procedure LoadFromStrings(const Lines: TStrings);
      procedure LoadFromString(const Line: string);
      procedure LoadFromNameValue(const Name, Value: string);
      function LoadFromCommandLine(out BadParamIndex: integer; out Msg: string): boolean;
      function CommandLineSyntax(const EOLN: string = #13#10): string;
      property CalibrePath: string read fCalibrePath write fCalibrePath;
      property GenerateMetadata: boolean read fGenerateMetadata write fGenerateMetadata;
      property RefreshExisting: boolean read fRefreshExisting write fRefreshExisting;
      property MemoryType: TReaderMemoryType read fMemoryType write fMemoryType;
      property Recursive: boolean read fRecursive write fRecursive;
      property FileNameRegExp: string read fFileNameRegexp write fFileNameRegexp;
      property PlaylistDefinitionFileName: string read fPlaylistFN write fPlaylistFN;
      property CacheInFileName: string read fCacheInFN write fCacheInFN;
      property CacheOutFileName: string read fCacheOutFN write fCacheOutFN;
      property BooksPath: string read fBooksPath write fBooksPath;
      property BooksBasePath: string read fBooksBasePath write fBooksBasePath;
      property TimeDelta: TDateTime read fTimeDelta write fTimeDelta;
      property CollectionRegexp: string read fCollectionRegexp write fCollectionRegexp;
      property CollectionRegexpTitle: string read fCollectionRegexpTitle write fCollectionRegexpTitle;
      property LRFMeta: string read fLRFMeta write fLRFMeta;
      property PDFMeta: string read fPDFMeta write fPDFMeta;
      property RTFMeta: string read fRTFMeta write fRTFMeta;
      property TXTMeta: string read fTXTMeta write fTXTMeta;
      property EPUBMeta: string read fEPUBMeta write fEPUBMeta;
      property ConvertToUTF: boolean read fConvertToUTF write fConvertToUTF;
      property Encoding: TEncoding read fEncoding write fEncoding;
    end;

implementation

const
  DEFAULT_BOOKSREGEXP = '\.(lrf|pdf|txt|rtf|epub)$';

  MEMORY_MEMORYSTICK = 'ms';
  MEMORY_SECUREDIGITAL = 'sd';
  MEMORY_INTERNAL = 'int';

{ TConfig }

constructor TConfig.Create;
begin
  Init;
end;

function TConfig.GetMemoryType(const MemType: string; Default: TReaderMemoryType): TReaderMemoryType;
begin
  if AnsiCompareText(MemType, MEMORY_MEMORYSTICK) = 0 then
    Result := mtMS
  else if AnsiCompareText(MemType, MEMORY_SECUREDIGITAL) = 0 then
    Result := mtSD
  else if AnsiCompareText(MemType, MEMORY_INTERNAL) = 0 then
    Result := mtInternal
  else
    Result := Default;
end;

procedure TConfig.Init;
begin
  CalibrePath := '';
  GenerateMetadata := False;
  RefreshExisting := False;
  MemoryType := mtInternal;
  Recursive := False;
  FileNameRegExp := DEFAULT_BOOKSREGEXP;
  PlaylistDefinitionFileName := '';
  CacheInFileName := '';
  CacheOutFileName := '';
  BooksPath := '';
  BooksBasePath := '';
  TimeDelta := 0;
  CollectionRegexp := '';
  CollectionRegexpTitle := '$0';
  LRFMeta := 'lrf-meta.exe %FILENAME%';
  PDFMeta := 'pdf-meta.exe %FILENAME%';
  RTFMeta := '';
  TXTMeta := '';
  EPUBMeta := 'epub-meta.exe %FILENAME%';
  ConvertToUTF := False;
  Encoding := enOEM;
end;

function TConfig.CommandLineSyntax(const EOLN: string): string;
begin
  Result := ''
          + 'Syntax: ' + ExtractFileName(ParamStr(0)) + ' [params] [playlist] [params] [books] [params]' + EOLN + EOLN
          + '   [playlist] ...... file name of a playlist definition file.' + EOLN
          + '   [books] ......... path to a directory where the books are stored.' + EOLN
          + '   [params] ........ one or more of:' + EOLN
          + '      -r ........... scan [books] recursively (including subdirectories).' + EOLN
          + '      -ms .......... create cache for MemoryStick.' + EOLN
          + '      -sd .......... create cache for SD card.' + EOLN
          + '      -int ......... create cache for internal memory.' + EOLN
          + '      -re <regex> .. only files matching regular expression <regex> will be' + EOLN
          + '                     considered. Default value is ' + DEFAULT_BOOKSREGEXP + EOLN
          + '      -c <file> .... file name of input cache file (if not used, the value' + EOLN
          + '                     from <playlist> is used).' + EOLN
          + '      -co <file> ... file name of output cache file (if not used, the value' + EOLN
          + '                     from <playlist> is used).' + EOLN
          + '      -d <num> ..... time difference between computer and reader is <num> hours' + EOLN
          + '      -cr <regex> .. files matching this regular expression will be moved into' + EOLN
          + '                     special "unlisted" collections.' + EOLN
          + '      -crt <str> ... title of the "unlisted" collection. May contain references' + EOLN
          + '                     to subexpressions of -cr in the form of $n where "n" is' + EOLN
          + '                     the number of subexpression. See TRegExpr.Substitute.' + EOLN
          + EOLN
          + '   Note: Either [playlist] or -cr must be present.' + EOLN
          + '';
end;

function TConfig.LoadFromCommandLine(out BadParamIndex: integer; out Msg: string): boolean;
var i, n: integer;
    s: string;
begin
  Result := True;
  BadParamIndex := 0;
  Msg := '';
  i := 1;
  n := ParamCount;
  while i <= n do
    begin
    s := ParamStr(i);
    Inc(i);
    if s <> '' then
      begin
      if s[1] = '-' then
        begin
        if AnsiCompareText(s, '-r') = 0 then
          Recursive := True
        else if AnsiCompareText(s, '-' + MEMORY_MEMORYSTICK) = 0 then
          MemoryType := mtMS
        else if AnsiCompareText(s, '-' + MEMORY_SECUREDIGITAL) = 0 then
          MemoryType := mtSD
        else if AnsiCompareText(s, '-' + MEMORY_INTERNAL) = 0 then
          MemoryType := mtInternal
        else if AnsiCompareText(s, '-re') = 0 then
          begin
          if i <= ParamCount then
            begin
            FileNameRegexp := ParamStr(i);
            Inc(i);
            end
          else
            begin
            Result := False;
            BadParamIndex := Pred(i);
            Msg := 'Missing -re value.';
            Exit;
            end;
          end
        else if AnsiCompareText(s, '-c') = 0 then
          begin
          if i <= ParamCount then
            begin
            CacheInFileName := ParamStr(i);
            Inc(i);
            {$IFNDEF ALLOW_NONEXISTENT_CACHE}
              if not FileExists(CacheInFileName) then
                begin
                Result := False;
                BadParamIndex := i;
                Msg := 'Input cache does not exist.';
                Exit;
                end;
            {$ENDIF}
            end
          else
            begin
            Result := False;
            BadParamIndex := Pred(i);
            Msg := 'Missing -c value.';
            Exit;
            end;
          end
        else if AnsiCompareText(s, '-co') = 0 then
          begin
          if i <= ParamCount then
            begin
            CacheOutFileName := ParamStr(i);
            Inc(i);
            end
          else
            begin
            Result := False;
            BadParamIndex := Pred(i);
            Msg := 'Missing -co value.';
            Exit;
            end;
          end
        else if AnsiCompareText(s, '-cr') = 0 then
          begin
          if i <= ParamCount then
            begin
            CollectionRegexp := ParamStr(i);
            Inc(i);
            end
          else
            begin
            Result := False;
            BadParamIndex := Pred(i);
            Msg := 'Missing -cr value.';
            Exit;
            end;
          end
        else if AnsiCompareText(s, '-crt') = 0 then
          begin
          if i <= ParamCount then
            begin
            CollectionRegexpTitle := ParamStr(i);
            Inc(i);
            end
          else
            begin
            Result := False;
            BadParamIndex := Pred(i);
            Msg := 'Missing -crt value.';
            Exit;
            end;
          end
        else if AnsiCompareText(s, '-?') = 0 then
          begin
          Result := False;
          BadParamIndex := Pred(i);
          Msg := 'Help requested.';
          Exit;
          end
        else
          begin
          Result := False;
          BadParamIndex := Pred(i);
          Msg := 'Unknown parameter.';
          Exit;
          end;
        end
      else if PlaylistDefinitionFileName = '' then
        if DirectoryExists(ExcludeTrailingBackslash(s)) then // Directory
          BooksPath := s
        else
          PlaylistDefinitionFileName := s
      else if BooksPath = '' then
        BooksPath := s
      else
        begin
        Result := False;
        BadParamIndex := Pred(i);
        Msg := 'Unknown parameter.';
        Exit;
        end;
      end;
    end;
end;

procedure TConfig.LoadFromIniFile(const IniFileName, Section: string);
var EncodingStr: string;
begin
  with TIniFile.Create(IniFileName) do
    try
      CalibrePath := ReadString(Section, 'CalibrePath', CalibrePath);
      GenerateMetadata := ReadBool(Section, 'GenerateMetadata', GenerateMetadata);
      RefreshExisting := ReadBool(Section, 'RefreshExisting', RefreshExisting);
      MemoryType := GetMemoryType(ReadString(Section, 'MemoryType', ''), MemoryType);
      Recursive := ReadBool(Section, 'Recursive', Recursive);
      FileNameRegExp := ReadString(Section, 'FileNameRegExp', FileNameRegexp);
      PlaylistDefinitionFileName := ReadString(Section, 'Playlist', PlaylistDefinitionFileName);
      CacheInFileName := ReadString(Section, 'Cache', CacheInFileName);
      CacheOutFileName := ReadString(Section, 'Cache.out', CacheOutFileName);
      BooksPath := ReadString(Section, 'BooksPath', BooksPath);
      BooksBasePath := ReadString(Section, 'BooksBasePath', BooksBasePath);
      TimeDelta := ReadFloat(Section, 'TimeDelta', TimeDelta);
      CollectionRegexp := ReadString(Section, 'CollectionRegExp', CollectionRegexp);
      CollectionRegexpTitle := ReadString(Section, 'CollectionRegExpTitle', CollectionRegexpTitle);
      LRFMeta := ReadString(Section, 'LRFMeta', LRFMeta);
      PDFMeta := ReadString(Section, 'PDFMeta', PDFMeta);
      RTFMeta := ReadString(Section, 'RTFMeta', RTFMeta);
      TXTMeta := ReadString(Section, 'TXTMeta', TXTMeta);
      EPUBMeta := ReadString(Section, 'EPUBMeta', EPUBMeta);
      ConvertToUTF := ReadBool(Section, 'ConvertMetadataToUTF', ConvertToUTF);
      EncodingStr := ReadString(Section, 'Encoding', '');
      if EncodingStr <> '' then
        LoadFromNameValue('Encoding', EncodingStr);
    finally
      Free;
      end;
end;

procedure TConfig.LoadFromNameValue(const Name, Value: string);
begin
  if Name = '' then
    Exit
  else if AnsiCompareText(Name, 'CalibrePath') = 0 then
    CalibrePath := Value
  else if AnsiCompareText(Name, 'GenerateMetadata') = 0 then
    GenerateMetadata := StrToIntDef(Value, 0) <> 0
  else if AnsiCompareText(Name, 'RefreshExisting') = 0 then
    RefreshExisting := StrToIntDef(Value, 0) <> 0
  else if AnsiCompareText(Name, 'MemoryType') = 0 then
    MemoryType := GetMemoryType(Value, MemoryType)
  else if AnsiCompareText(Name, 'Recursive') = 0 then
    Recursive := StrToIntDef(Value, 0) <> 0
  else if AnsiCompareText(Name, 'FileNameRegExp') = 0 then
    FileNameRegExp := Value
  else if AnsiCompareText(Name, 'Playlist') = 0 then
    PlaylistDefinitionFileName := Value
  else if AnsiCompareText(Name, 'Cache') = 0 then
    CacheInFileName := Value
  else if AnsiCompareText(Name, 'Cache.out') = 0 then
    CacheOutFileName := Value
  else if AnsiCompareText(Name, 'BooksPath') = 0 then
    BooksPath := Value
  else if AnsiCompareText(Name, 'BooksBasePath') = 0 then
    BooksBasePath := Value
  else if AnsiCompareText(Name, 'TimeDelta') = 0 then
    TimeDelta := StrToFloat(Value)
  else if AnsiCompareText(Name, 'CollectionRegExp') = 0 then
    CollectionRegexp := Value
  else if AnsiCompareText(Name, 'CollectionRegExpTitle') = 0 then
    CollectionRegexpTitle := Value
  else if AnsiCompareText(Name, 'LRFMeta') = 0 then
    LRFMeta := Value
  else if AnsiCompareText(Name, 'PDFMeta') = 0 then
    PDFMeta := Value
  else if AnsiCompareText(Name, 'RTFMeta') = 0 then
    RTFMeta := Value
  else if AnsiCompareText(Name, 'TXTMeta') = 0 then
    TXTMeta := Value
  else if AnsiCompareText(Name, 'EPUBMeta') = 0 then
    EPUBMeta := Value
  else if AnsiCompareText(Name, 'ConvertMetadataToUTF') = 0 then
    ConvertToUTF := StrToIntDef(Value, 0) <> 0
  else if AnsiCompareText(Name, 'Encoding') = 0 then
    begin
    if AnsiCompareText(Value, 'OEM') = 0 then
      Encoding := enOEM
    else if AnsiCompareText(Value, 'ANSI') = 0 then
      Encoding := enANSI
    else if AnsiCompareText(Value, 'Unicode') = 0 then
      Encoding := enUnicode
    else if AnsiCompareText(Value, 'UTF8') = 0 then
      Encoding := enUTF8
    else if AnsiCompareText(Value, 'UTF-8') = 0 then
      Encoding := enUTF8
      ;
    end
  else
    ;
end;

procedure TConfig.LoadFromString(const Line: string);
var i: integer;
    Name, Value: string;
begin
  if Line <> '' then
    begin
    i := Pos('=', Line);
    if i > 0 then
      begin
      Name := Trim(Copy(Line, 1, Pred(i)));
      Value := Trim(Copy(Line, Succ(i), MaxInt));
      end
    else
      begin
      Name := Line;
      Value := '';
      end;
    LoadFromNameValue(Name, Value);
    end;
end;

procedure TConfig.LoadFromStrings(const Lines: TStrings);
var i: integer;
begin
  for i := 0 to Pred(Lines.Count) do
    LoadFromString(Lines[i]);
end;

end.
